/***************************************************************************
 *            splashy_config-main.c
 *
 *  Sun Sep 11 17:37:34 2005
 *  Copyright  2005  Vincent Amouret
 *  vincent.amouret@gmail.com
 *  Luis Mondesi <lemsx1@gmail.com> 
 ****************************************************************************/
/*
 * This file is part of Splashy.
 * 
 * Splashy is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any
 * later version.
 * 
 * Splashy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with Splashy; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301 USA 
 */
/*
 * NOTE TO DEVELOPERS:
 * Please keep in mind that the interface (cli) for splashy_config is very
 * volatile at this moment. When writing front-ends for splashy_config, 
 * please use the long version of the options and not the short ones:
 * i.e. --install-theme, --textarea-border-show... etc..
 * 2006-02-26 16:14 EST - Luis Mondesi <lemsx1@gmail.com> 
 */

#include <unistd.h>		/* getuid */
#include <getopt.h>		/* getopt_long */
#include <glib/gi18n.h>

#include "common_macros.h"
#include "common_functions.h"
#include "splashy_config-functions.h"

#define PROGNAME "splashy_config"
#define HELP \
_("------------------\n\
SPLASHY-CONFIG 0.1\n\
------------------\n\
A Splashy Configuration Tool\n\
\n\
usage: %s [option] arg \n\
\n\
-h, --help                          Display this help\n\
-g, --get-key XPATH                 Returns the value of a key from Splashy config file, or the current theme\n\
-s, --set-theme THEME               Set THEME as the Splashy theme\n\
-i, --install-theme THEME.tar.gz    Install the THEME from a tarball\n\
-r, --remove-theme THEME            Remove the THEME\n\
    --info                          Display some information\n\
-c, --create-theme  [args...]       Create a theme in an interactive mode if no arguments\n\
                                    Inline mode arguments (this mode must be used with caution)\n\
            --name  NAME                 Set the NAME of the new Splashy theme\n\
            --version VERSION            Set the VERSION of the new Splashy theme\n\
            --description DESC           Set the DESC of the new Splashy theme\n\
            --urls URLs                  Set the related URLs for this theme\n\
            --author AUTHOR              Set the AUTHOR(s) of the new Splashy theme\n\
            --boot-image PATH            Upload the boot image located at the specified PATH\n\
            --shutdown-image PATH        Upload the shutdown image located at the specified PATH\n\
            --error-image PATH           Upload the error image located at the specified PATH\n\
            --background-width WIDTH     Set the WIDTH of the background\n\
            --background-height HEIGHT   Set the HEIGHT of the background\n\
            --progressbar-x X-POSITION   Set the X-POSITION of the progress bar (percentage if the background height and width are set, else pixel units)\n\
            --progressbar-y Y-POSITION   Set the Y-POSITION of the progress bar (percentage if the background height and width are set, else pixel units)\n\
            --progressbar-width WIDTH    Set the WIDTH of the progress bar      (percentage if the background height and width are set, else pixel units)\n\
            --progressbar-height HEIGHT  Set the HEIGHT of the progress bar     (percentage if the background height and width are set, else pixel units)\n\
            --progressbar-red RED        Set the RED channel of the progress bar (0->255)\n\
            --progressbar-green GREEN    Set the GREEN channel of the progress bar (0->255)\n\
            --progressbar-blue BLUE      Set the BLUE channel of the progress bar (0->255)\n\
            --progressbar-alpha ALPHA    Set the ALPHA channel of the progress bar (0->255)\n\
            --progressbar-border-show yes|no  Set a border around the progressbar\n\
            --progressbar-border-red RED  Set the RED channel of the border around the progressbar\n\
            --progressbar-border-green GREEN  Set the GREEN channel of the border around the progressbar\n\
            --progressbar-border-blue BLUE  Set the BLUE channel of the border around the progressbar\n\
            --progressbar-border-alpha ALPHA  Set the ALPHA channel of the border around the progressbar\n\
            --textbox-show yes|no      Set a text box\n\
            --textbox-x X-POS          Set the X-POS of the text box (percentage if the background height and width are set, else pixel units)\n\
            --textbox-y Y-POS          Set the Y-POS of the text box (percentage if the background height and width are set, else pixel units)\n\
            --textbox-width WIDTH      Set the WIDTH of the text box (percentage if the background height and width are set, else pixel units)\n\
            --textbox-height HEIGHT    Set the HEIGHT of the text area (percentage if the background height and width are set, else pixel units)\n\
            --textbox-red RED          Set the RED channel of the text area (0->255)\n\
            --textbox-green GREEN      Set the GREEN channel of the text area (0->255)\n\
            --textbox-blue BLUE        Set the BLUE channel of the text area (0->255)\n\
            --textbox-alpha ALPHA      Set the ALPHA channel of the text area (0->255)\n\
            --textbox-border-show yes|no  Set a border around the text area\n\
            --textbox-border-red RED      Set the RED channel of the border of the text area (0->255)\n\
            --textbox-border-green GREEN  Set the GREEN channel of the border of the text area (0->255)\n\
            --textbox-border-blue BLUE    Set the BLUE channel of the border of the text area (0->255)\n\
            --textbox-border-alpha ALPHA  Set the ALPHA channel of the border of the text area (0->255)\n\
            --text-font-file FILE        Upload the FONT file\n\
            --text-font-height HEIGHT    Set the HEIGHT of the text area font\n\
            --text-font-red RED          Set the RED channel of the text area font (0->255)\n\
            --text-font-green GREEN      Set the GREEN channel of the text area font (0->255)\n\
            --text-font-blue BLUE        Set the BLUE channel of the text area font (0->255)\n\
            --text-font-alpha ALPHA      Set the ALPHA channel of the text area font (0->255)\n\
            --autoverbose yes|no         Set if Splashy turn into verbose mode when an error appears at boot\n\
            --progressbaronboot yes|no   Set if Splashy shows a progress bar on boot \n\
            --progressbaronshutdown yes|no Set if Splashy shows a progress bar on shutdown \n\
            --fadein yes|no              Set if Splashy appears smoothly\n\
            --fadeout yes|no             Set if Splashy disappears smoothly\n")
#define USAGE \
_("usage: %s [option] arg \n\
[-h, --help][-s, --set-theme THEME][-i, --install-theme THEME.tar.gz]\n\
[-r, --remove-theme THEME][--info][-c, --create-theme  [args...]][-g, --get-key XPATH]\n\
See --help for more information\n")

static struct option long_options[] = {
	{"set-theme", required_argument, 0, 's'},
	{"install-theme", required_argument, 0, 'i'},
	{"remove-theme", required_argument, 0, 'r'},
	{"info", no_argument, 0, 'a'},
	{"create-theme", optional_argument, 0, 'c'},
	//{"upload-image", required_argument, 0, 'u'},
	{"help", no_argument, 0, 'h'},
	{"get-key", required_argument, 0, 'k'},
	{0, 0, 0, 0}
};
static struct option create_options[] = {
	{"name", required_argument, 0, 'a'},
	{"version", required_argument, 0, 'b'},
	{"description", required_argument, 0, 'c'},
	{"urls", required_argument, 0, 'd'},
	{"author", required_argument, 0, 'e'},
	{"progressbar-x", required_argument, 0, 'f'},
	{"progressbar-y", required_argument, 0, 'g'},
	{"progressbar-width", required_argument, 0, 'h'},
	{"progressbar-height", required_argument, 0, 'i'},
	{"progressbar-red", required_argument, 0, 'j'},
	{"progressbar-green", required_argument, 0, 'k'},
	{"progressbar-blue", required_argument, 0, 'l'},
	{"progressbar-alpha", required_argument, 0, 'm'},
	{"progressbar-border-show", required_argument, 0, 'n'},
	{"progressbar-border-red", required_argument, 0, 'o'},
	{"progressbar-border-green", required_argument, 0, 'p'},
	{"progressbar-border-blue", required_argument, 0, 'q'},
	{"progressbar-border-alpha", required_argument, 0, 'r'},
	{"boot-image", required_argument, 0, 's'},
	{"shutdown-image", required_argument, 0, 't'},
	{"error-image", required_argument, 0, 'u'},
	{"background-width", required_argument, 0, 'v'},
	{"background-height", required_argument, 0, 'w'},
	{"textbox-show", required_argument, 0, 'x'},
	{"textbox-x", required_argument, 0, 'y'},
	{"textbox-y", required_argument, 0, 'z'},
	{"textbox-width", required_argument, 0, 'A'},
	{"textbox-height", required_argument, 0, 'B'},
	{"textbox-red", required_argument, 0, 'C'},
	{"textbox-green", required_argument, 0, 'D'},
	{"textbox-blue", required_argument, 0, 'E'},
	{"textbox-alpha", required_argument, 0, 'F'},
	{"textbox-border-show", required_argument, 0, 'G'},
	{"textbox-border-red", required_argument, 0, 'H'},
	{"textbox-border-green", required_argument, 0, 'I'},
	{"textbox-border-blue", required_argument, 0, 'J'},
	{"textbox-border-alpha", required_argument, 0, 'K'},
	{"text-font-file", required_argument, 0, 'L'},
	{"text-font-height", required_argument, 0, 'M'},
	{"text-font-red", required_argument, 0, 'N'},
	{"text-font-green", required_argument, 0, 'O'},
	{"text-font-blue", required_argument, 0, 'P'},
	{"text-font-alpha", required_argument, 0, 'Q'},
	{"autoverbose", required_argument, 0, 'R'},
	{"progressbaronboot", required_argument, 0, 'S'},
	{"progressbaronshutdown", required_argument, 0, 'T'},
	{"fadein", required_argument, 0, 'U'},
	{"fadeout", required_argument, 0, 'V'},
	{0, 0, 0, 0}
};

gint
main (gint argc, gchar * argv[])
{
	if (!splashy_init (FALSE))
	{
		ERROR_PRINT ("%s",
			     "Error occured while starting Splashy\n"
			     "Make sure that you can read Splashy's configuration file\n");
		/*
		 * g_free() memory 
		 */
		return 1;
	}
	g_set_prgname (PROGNAME);

	XmlFields *inline_theme = g_new0 (XmlFields, 1);
	gint retopt = 0;
	gint create_retopt = 0;
	gint option_index = 0;
	gint create_option_index = 0;
	retopt = getopt_long (argc, argv, "hs:i:r:ic::k:g:", long_options,
			      &option_index);
	if (retopt == -1)
	{
		printf (_("Missing arguments\n"));
		printf (USAGE, g_get_prgname ());
		return 0;
	}
	switch (retopt)
	{
	case 's':
		set_new_theme (optarg);
		break;
	case 'i':
		install_theme (optarg);
		break;
	case 'r':
		remove_theme (optarg);
		break;
	case 'a':
		information ();
		break;
	case 'c':
		if (argv[optind] == NULL)	// interactive mode */
			/* create_theme (get_fields ()); */
			g_print (" NOT IMPLEMENTED \n");
		else		// inline mode
		{

			while (argc > optind)
			{

				create_retopt =
					getopt_long_only (argc, argv, "",
							  create_options,
							  &create_option_index);
				switch (create_retopt)
				{
				case 'a':
					inline_theme->name =
						g_strdup (optarg);
					break;
				case 'b':
					inline_theme->version =
						g_strtod (optarg, 0);
					break;
				case 'c':
					inline_theme->description =
						g_strdup (optarg);
					break;
				case 'd':
					inline_theme->urls =
						g_strdup (optarg);
					break;
				case 'e':
					inline_theme->author =
						g_strdup (optarg);
					break;
				case 'f':
					inline_theme->pb_x =
						g_strtod (optarg, 0);
					break;
				case 'g':
					inline_theme->pb_y =
						g_strtod (optarg, 0);
					break;
				case 'h':
					inline_theme->pb_width =
						g_strtod (optarg, 0);
					break;
				case 'i':
					inline_theme->pb_height =
						g_strtod (optarg, 0);
					break;
				case 'j':
					inline_theme->pb_red =
						g_strtod (optarg, 0);
					break;
				case 'k':
					inline_theme->pb_green =
						g_strtod (optarg, 0);
					break;
				case 'l':
					inline_theme->pb_blue =
						g_strtod (optarg, 0);
					break;
				case 'm':
					inline_theme->pb_alpha =
						g_strtod (optarg, 0);
					break;
				case 'n':
					inline_theme->pb_border_show =
						g_strdup (optarg);
					break;
				case 'o':
					inline_theme->pb_border_red =
						g_strtod (optarg, 0);
					break;
				case 'p':
					inline_theme->pb_border_green =
						g_strtod (optarg, 0);
					break;
				case 'q':
					inline_theme->pb_border_blue =
						g_strtod (optarg, 0);
					break;
				case 'r':
					inline_theme->pb_border_alpha =
						g_strtod (optarg, 0);
					break;
				case 's':
					inline_theme->bg_boot =
						g_strdup (optarg);
					break;
				case 't':
					inline_theme->bg_shutdown =
						g_strdup (optarg);
					break;
				case 'u':
					inline_theme->bg_error =
						g_strdup (optarg);
					break;
				case 'v':
					inline_theme->bg_width =
						g_strtod (optarg, 0);
					break;
				case 'w':
					inline_theme->bg_height =
						g_strtod (optarg, 0);
					break;
				case 'x':
					inline_theme->textbox_show =
						g_strdup (optarg);
					break;
				case 'y':
					inline_theme->textbox_x =
						g_strtod (optarg, 0);
					break;
				case 'z':
					inline_theme->textbox_y =
						g_strtod (optarg, 0);
					break;
				case 'A':
					inline_theme->textbox_width =
						g_strtod (optarg, 0);
					break;
				case 'B':
					inline_theme->textbox_height =
						g_strtod (optarg, 0);
					break;
				case 'C':
					inline_theme->textbox_red =
						g_strtod (optarg, 0);
					break;
				case 'D':
					inline_theme->textbox_green =
						g_strtod (optarg, 0);
					break;
				case 'E':
					inline_theme->textbox_blue =
						g_strtod (optarg, 0);
					break;
				case 'F':
					inline_theme->textbox_alpha =
						g_strtod (optarg, 0);
					break;
				case 'G':
					inline_theme->textbox_border_show =
						g_strdup (optarg);
					break;
				case 'H':
					inline_theme->textbox_border_red =
						g_strtod (optarg, 0);
					break;
				case 'I':
					inline_theme->textbox_border_green =
						g_strtod (optarg, 0);
					break;
				case 'J':
					inline_theme->textbox_border_blue =
						g_strtod (optarg, 0);
					break;
				case 'K':
					inline_theme->textbox_border_alpha =
						g_strtod (optarg, 0);
					break;
				case 'L':
					inline_theme->textfont_file =
						g_strdup (optarg);
					break;
				case 'M':
					inline_theme->textfont_height =
						g_strtod (optarg, 0);
					break;
				case 'N':
					inline_theme->textfont_red =
						g_strtod (optarg, 0);
					break;
				case 'O':
					inline_theme->textfont_green =
						g_strtod (optarg, 0);
					break;
				case 'P':
					inline_theme->textfont_blue =
						g_strtod (optarg, 0);
					break;
				case 'Q':
					inline_theme->textfont_alpha =
						g_strtod (optarg, 0);
					break;
				case 'R':
					inline_theme->verbose =
						g_strdup (optarg);
					break;
				case 'S':
					inline_theme->progressbaronboot =
						g_strdup (optarg);
					break;
				case 'T':
					inline_theme->progressbaronshutdown =
						g_strdup (optarg);
					break;
				case 'U':
					inline_theme->fadein =
						g_strdup (optarg);
					break;
				case 'V':
					inline_theme->fadeout =
						g_strdup (optarg);
					break;
				}
			}
			create_theme (inline_theme);

			/*
			 * memory free up 
			 */

			if (inline_theme->name)
				g_free (inline_theme->name);
			if (inline_theme->description)
				g_free (inline_theme->description);
			if (inline_theme->urls)
				g_free (inline_theme->urls);
			if (inline_theme->author)
				g_free (inline_theme->author);
			if (inline_theme->pb_border_show)
				g_free (inline_theme->pb_border_show);
			if (inline_theme->bg_boot)
				g_free (inline_theme->bg_boot);
			if (inline_theme->bg_shutdown)
				g_free (inline_theme->bg_shutdown);
			if (inline_theme->bg_error)
				g_free (inline_theme->bg_error);
			if (inline_theme->textbox_show)
				g_free (inline_theme->textbox_show);
			if (inline_theme->textbox_border_show)
				g_free (inline_theme->textbox_border_show);
			if (inline_theme->textfont_file)
				g_free (inline_theme->textfont_file);
			if (inline_theme->verbose)
				g_free (inline_theme->verbose);
			if (inline_theme->progressbaronboot)
				g_free (inline_theme->progressbaronboot);
			if (inline_theme->progressbaronshutdown)
				g_free (inline_theme->progressbaronshutdown);
			if (inline_theme->fadein)
				g_free (inline_theme->fadein);
			if (inline_theme->fadeout)
				g_free (inline_theme->fadeout);

		}
		break;
		//case 'u':
		//      copy_file (optarg, argv[optind]);
		//      break;
	case 'g':
	case 'k':
		g_print ("%s", xml_parser_get_text (optarg));
		break;
	case 'h':
		printf (HELP, g_get_prgname ());
		break;
	case '?':
		printf (USAGE, g_get_prgname ());
		break;
	}

	return (0);
}
