/************************************************************************
 *                              splashy_update.c                        *
 *                                                                      *
 *  2006-02-25 13:17 EST                                                *
 *  Copyright  2006     Andrew Williams <mistik1@geeksinthehood.net>    *
 *                                                                      *
 ************************************************************************/
/*
 * This file is part of Splashy.
 * 
 * Splashy is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any
 * later version.
 * 
 * Splashy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with Splashy; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301 USA 
 */

/*
 * Andrew Williams <mistik1@geeksinthehood.net>
 * This is a simple attempt to replace using echo 
 * write to the Splashy named pipe.
 * This will write to the pipe in a non blocking fashion so init
 * scripts dont get hung up if nothing is reading from the pipe
 */

#include <string.h> /* strerror */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <glib.h>
#include <errno.h> /* errno */

#include "common_macros.h"
/* #include "common_functions.h" */
#include "xml_parser.h"
#include "xml_config.h"

int
main (int argc, char **argv)
{
        int splashy_pipe, len,result;

        //fprintf(stderr,"splashy_update doing %s\n",argv[1]);
        
        if (argc != 2)
        {
                ERROR_PRINT ("%s","Invalid number of arguments");
                ERROR_PRINT ("%s","This program accepts a single argument");
                exit (3);
        }
        
        if (!init_config (SPL_CONFIG_FILE))
        {
                ERROR_PRINT ("%s",
                                "Error occured while starting splashy_update. Could not read configuration\n");
                return 3;
        }

        splashy_pipe = open (xml_parser_get_text (SPL_FIFO_XPATH), O_WRONLY | O_NONBLOCK);

        /*
         * This is the more "canonical" way to code this.
         */

        if (splashy_pipe < 0)
                return (1);     /* fifo does not exist */

        len = strlen (argv[1]) + 1;
        result = write (splashy_pipe, argv[1], len);

        if (result < len)        /* check both short write and 
                                                 * fail */
        {
                if ( errno == EAGAIN )
                {
                        ERROR_PRINT ("%s","splashy_update: could not open FIFO for nonblock writing");
                        close (splashy_pipe);
                        return (4);
                }
                ERROR_PRINT ("%s",strerror(errno));
                return (2);
        }

        close (splashy_pipe);

        return 0;
}
