/*
 * This file is part of Splashy.
 * 
 * Splashy is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any
 * later version.
 * 
 * Splashy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with Splashy; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301 USA 
 */
#ifndef _video_H
#define _video_H

#include <glib.h>
#include <directfb.h>

typedef struct splashy_videomode_s
{
        gint width;
        gint height;
        gint out_width;
        gint out_height;
        gint overx;
        gint overy;
        gint bpp;
} splashy_videomode_t;

typedef struct
{
        gboolean enabled;
        IDirectFBSurface *offscreen;    /** off-screen storage */
        IDirectFBSurface *surface;      /** Subsurface to draw text on */
        DFBSurfaceDescription desc;
        DFBRectangle area;
        DFBFontDescription fontdesc;
} splashy_textbox_t;

typedef struct
{
        IDirectFB *dfb;                         /** our directfb object */
        IDirectFBImageProvider *provider;       /** our image class */
        IDirectFBWindow *primary_window;        /** our main window. where opacity takes place */
        IDirectFBSurface *primary_window_surface; /** interface to window's surface */

        IDirectFBDisplayLayer *primary_layer;   /** interface to our layer */
        DFBDisplayLayerConfig  primary_layer_config; /** configuration for our primary_layer */
        IDirectFBSurface *primary_surface;      /** our main background */
        IDirectFBInputDevice *keyboard;         /** our main input dev */
        IDirectFBEventBuffer *ev_buffer;        /** our events listener */
        splashy_videomode_t *mode;              /** video mode supported by surface */
        splashy_textbox_t *textbox;             /** text box parameters */
        DFBRectangle progressbar;               /** our progressbar */
        __u8 opacity;                           /** current opacity level */
} splashy_video_t;

/*
 * public functions 
 */
void
video_reset_progressbar_counters();
void video_set_progressbar_forward(gboolean go_forward);
gint video_draw_progressbar (splashy_video_t * video,
                             DFBRectangle * progressbar);
gint video_update_progressbar (splashy_video_t * video,
                               DFBRectangle * progressbar, gint perc);
void video_start_splash (splashy_video_t * video, const gchar * background);
void video_change_splash (splashy_video_t * video, const gchar * newimage, gboolean show_progress);
void video_start_text_area (splashy_video_t * video, gboolean show_progress);
void video_set_mode (splashy_video_t * video);
void video_allow_vt_switching ();
/*
 * allows to print a single message to the center of the textbox area
 * reseting the old one
 */
void video_printline (splashy_video_t * video, gchar * string, gboolean show_progress);
/*
 * allows to print a message appended to the end of the textbox area
 * keeping the old one
 */
void video_printline_s (splashy_video_t * video, gchar * string, gboolean show_progress);
/* 
 * write data to our textbox the right way. it uses video_printline_s() to do
 * the actual writing. @see verbose_text_loop() in splashy_functions.c
 */
void video_writelog (splashy_video_t * video, unsigned char *ptr, 
                gint len, char *ringbuf, char *outptr, char *endptr, 
                gboolean show_progress);

void video_fade_in (splashy_video_t * video);
void video_fade_out (splashy_video_t * video);

#endif
