/************************************************************************
 *                              xml_config.c                            *
 *                                                                      *
 *  2005-10-01 11:21 EDT                                                *
 *  Copyright  2005  Luis Mondesi <lemsx1@gmail.com>                    *
 ************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

/*
 * A very basic implementation of an XML configuration parser using xml_parser
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <glib.h>

#include "common_macros.h"
#include "common_functions.h"
#include "xml_parser.h"
#include "xml_config.h"

/**
 * @ingroup core
 * Initializes a our XML db using _filename as main config file.
 * It will determine our theme config file from this file.
 * @param _filename filename path to read
 * @return true on success
 */
gboolean
init_config (const gchar * _filename)
{
        DEBUG_PRINT ("ENTERING init_config(%s)", _filename);
        /*
         * sanity check 
         */
        if (splashy_xml_config != NULL) /* splashy_xml_config has been
                                         * initialized */
                return TRUE;
        /*
         * slurp our config file to splashy_xml_config 
         */
        gboolean ret = xml_parser_init (_filename);
        if (ret == TRUE)
        {
                splashy_xml_config = xml_parser_get_db ();
                /*
                 * set default theme path now 
                 */
                GString *theme_path = g_string_new (splashy_get_theme_dir ());
                g_string_append (theme_path,
                                 G_DIR_SEPARATOR_S
                                 SPL_THEME_CONFIG_FILE_NAME);

                DEBUG_PRINT ("init_config(): constructed theme path <<%s>>",
                             theme_path->str);
                /*
                 * now the good part, if this SPL_THEME_CONFIG_FILE_NAME file doesn't 
                 * exist at the current path, we need to fall back to our default file
                 * SPL_DEFAULT_THEME in SPL_CONFIG_DIR+SPL_CONFIG_FILE_NAME 
                 * (old: /etc/splashy/config.xml) must have
                 * a full path to the fallback theme folder
                 */
                if (!g_file_test (theme_path->str, G_FILE_TEST_IS_REGULAR))
                {
                        g_string_free (theme_path, TRUE);
                        theme_path = g_string_new ("");
                        /*
                         * assumes SPL_DEFAULT_THEME has a full path to the
                         * theme 
                         */
                        g_string_append (theme_path,
                                         xml_parser_get_text
                                         (SPL_DEFAULT_THEME));
                        g_string_append (theme_path,
                                         G_DIR_SEPARATOR_S
                                         SPL_THEME_CONFIG_FILE_NAME);
                }
                DEBUG_PRINT ("init_config(): attempting to read theme <<%s>>",
                             theme_path->str);

                /*
                 * append tags from our theme file to our main XML db 
                 */
                xml_parser_set_xml_file (theme_path->str);
                if (!xml_parser_read (splashy_xml_config))
                {
                        DEBUG_PRINT
                                ("init_config(): failed to init themes %s",
                                 "");
                        return FALSE;
                }
        }
        return ret;
}

/*
 * EOF 
 */
